unit ChatClass;
interface
uses DB, ADODB, Dialogs, SysUtils;

Type
        TChat = class (TObject)
       private
          ADOConnection                 : TADOConnection; //
          ADOQuery                      : TADOQuery;      //   
          ConnectStr                    : string;
          ChatDataBasePublic            : string;         //   ( )
          ChatDataBasePrivate           : string;         //   ()
          MaxPageSizeForBasePublic      : Integer;        //.   ( )
          MaxPageSizeForBasePrivate     : Integer;        //.   ( )
        procedure Update; overload;
        procedure Delete; overload;
        procedure Update(UserID : Integer); overload;
        procedure Delete(UserID : Integer); overload;
       public
        constructor Create;
        destructor Destroy; override;
        procedure SentMessagePublic  (Message : string);  //c  ( )
        procedure SentMessagePrivate (Message : string; UserID : Integer); //c  ()
        function  GetMessagePublic   (MaxPageSizeForUser : Integer) : string; //  ( )
        function  GetMessagePrivate  (MaxPageSizeForUser : Integer; UserID : Integer) : string; //  ()
        procedure Open;
        procedure Close;
       published
        property ConnectionString : string read ConnectStr write ConnectStr;
        property DataBasePublic  : string read ChatDataBasePublic write ChatDataBasePublic;
        property DataBasePrivate : string read ChatDataBasePrivate write ChatDataBasePrivate;
        property PageSizeForBasePublic  : Integer read MaxPageSizeForBasePublic
                                                                write MaxPageSizeForBasePublic;
        property PageSizeForBasePrivate : Integer read MaxPageSizeForBasePrivate
                                                                write MaxPageSizeForBasePrivate;
end;

implementation

constructor TChat.Create;
begin
    ADOConnection         := TADOConnection.Create(nil);
    ADOQuery              := TADOQuery.Create(nil);
end;

destructor TChat.Destroy;
begin
       if ADOConnection <> nil then Close;
end;

procedure TChat.SentMessagePublic  (Message : string);
begin
        Update;
        Delete;
        ADOQuery.Close;
        ADOQuery.SQL.Clear;
        ADOQuery.SQL.Add('INSERT INTO '+ChatDataBasePublic+' (Message, MessagePosition) VALUES(?, ?)');
        ADOQuery.Parameters.Items[0].Value:=Message;
        ADOQuery.Parameters.Items[1].Value:=1;
        ADOQuery.ExecSQL;
end;

procedure TChat.SentMessagePrivate (Message : string; UserID : Integer);
begin
        Update(UserID);
        Delete(UserID);
        ADOQuery.Close;
        ADOQuery.SQL.Clear;
        ADOQuery.SQL.Add('INSERT INTO '+ ChatDataBasePrivate +' (Message, MessagePosition, UserID) VALUES(?, ?, ?)');
        ADOQuery.Parameters.Items[0].Value:=Message;
        ADOQuery.Parameters.Items[1].Value:=1;
        ADOQuery.Parameters.Items[2].Value:=UserID;
        ADOQuery.ExecSQL;
end;

function  TChat.GetMessagePublic   (MaxPageSizeForUser : Integer) : string;
var
        Temp    : Integer;
        Max     : Integer;
begin
        Result:='';
        ADOQuery.Close;
        ADOQuery.SQL.Clear;
        ADOQuery.SQL.Add('SELECT Message FROM '+ ChatDataBasePublic + ' ORDER BY MessagePosition');
        ADOQuery.Open;
        if MaxPageSizeForUser > ADOQuery.RecordCount then
                Max := ADOQuery.RecordCount
            else
                Max := MaxPageSizeForUser;
           for Temp:=1 to Max do
                begin
                        Result:=Result + ADOQuery.FieldByName('Message').AsString;
                        ADOQuery.Next;
                end;
end;

function  TChat.GetMessagePrivate  (MaxPageSizeForUser : Integer; UserID : Integer) : string;
var
        Temp    : Integer;
        Max     : Integer;
begin
        Result:='';
        ADOQuery.Close;
        ADOQuery.SQL.Clear;
        ADOQuery.SQL.Add('SELECT * FROM ' + ChatDataBasePrivate +
                                ' WHERE UserID='+ Inttostr(UserID) +' ORDER BY MessagePosition');
        ADOQuery.Open;
        if MaxPageSizeForUser > ADOQuery.RecordCount then
                Max := ADOQuery.RecordCount
            else
                Max := MaxPageSizeForUser;
           for Temp:=1 to Max do
                begin
                        Result:=Result + ADOQuery.FieldByName('Message').AsString;
                        ADOQuery.Next;
                end;
end;

procedure TChat.Open;
begin
        ADOConnection.ConnectionString:=ConnectStr;
        ADOConnection.Open;
        ADOQuery.Connection:=ADOConnection;
end;

procedure TChat.Update;
begin
        ADOQuery.Close;
        ADOQuery.SQL.Clear;
        ADOQuery.SQL.Add('UPDATE '+ChatDataBasePublic+' SET MessagePosition = MessagePosition + 1');
        ADOQuery.ExecSQL;
end;

procedure TChat.Delete;
begin
        ADOQuery.Close;
        ADOQuery.SQL.Clear;
        ADOQuery.SQL.Add('DELETE FROM '+ChatDataBasePublic+' WHERE MessagePosition>?');
        ADOQuery.Parameters.Items[0].Value:=MaxPageSizeForBasePublic;
        ADOQuery.ExecSQL;
end;

procedure TChat.Update(UserID : Integer);
begin
        ADOQuery.Close;
        ADOQuery.SQL.Clear;
        ADOQuery.SQL.Add('UPDATE '+ChatDataBasePrivate+' SET MessagePosition = MessagePosition + 1 WHERE UserID=?');
        ADOQuery.Parameters.Items[0].Value:=UserID;
        ADOQuery.ExecSQL;
end;

procedure TChat.Delete(UserID : Integer);
begin
        ADOQuery.Close;
        ADOQuery.SQL.Clear;
        ADOQuery.SQL.Add('DELETE FROM '+ChatDataBasePrivate+' WHERE UserID=? AND MessagePosition>?');
        ADOQuery.Parameters.Items[0].Value:=UserID;
        ADOQuery.Parameters.Items[1].Value:=MaxPageSizeForBasePrivate;
        ADOQuery.ExecSQL;
end;

procedure TChat.Close;
begin
    ADOQuery.Close;
        ADOQuery.Free;
    ADOConnection.Close;
        ADOConnection.Free;
end;

end.
